#include <pragma/noviasys_lib.h>
#include <pragma/exec_lib.h>
#include <pragma/dos_lib.h>
#include <dos/dos.h>
#include <stdio.h>
#include <novia/novia_dos.h>
#include <exec/lists.h>
#include <novia/novia_mail.h>

struct Library *NoviaSysBase;

struct INet_OutgoingMail *LoadMail(char *filename);

void ioprintf(const char *string, ...)
{
	char *buffer=AllocVec(10000,MEMF_ANY|MEMF_CLEAR);
	if (buffer)
	{
		vsprintf(buffer, string, unsigned int(&string + 1));
		Writeio(buffer,-1);
		FreeVec(buffer);
	}
}

void FreeMail(struct INet_OutgoingMail *mail)
{
	if (mail)
	{
		struct INet_SendMailHeaderEntry *entry;
		if (mail->totable)
			FreeVec(mail->totable);
		if (mail->header)
			FreeVec(mail->header);
		if (mail->text)
			FreeVec(mail->text);
		if (mail->to.mlh_Head)
		{
			while (!IsListEmpty((List *)&mail->to))
			{
				if ((entry = (INet_SendMailHeaderEntry *)mail->to.mlh_Head))
				{
					Remove((Node *)entry);
					if (entry->string)
						FreeVec(entry->string);
					FreeVec(entry);
				}
			}
		}
	}
}

void main()
{
	if ((NoviaSysBase = OpenLibrary("noviasys.library", 0)))
	{
		BPTR				lock;
		FileInfoBlock  *fib;
		struct 			INet_OutgoingMail *mail;
		char				buffer[100];
		if ((fib = AllocDosObject(DOS_FIB, 0)))
		{
			if ((lock = Lock("novia:uucp/outgoing", SHARED_LOCK)))
			{
				if ((Examine(lock, fib)))
				{
					while (ExNext(lock, fib))
					{
						sprintf(buffer, "novia:uucp/outgoing/%s",fib->fib_FileName);
						if ((mail = LoadMail(buffer)))
						{
							ioprintf("send mail: %s\n",buffer);
							FreeMail(mail);
						}
					}
				}
				UnLock(lock);
			}
			FreeDosObject(DOS_FIB, fib);
		}
		CloseLibrary(NoviaSysBase);
	}
}

struct INet_OutgoingMail *LoadMail(char *filename)
{
	struct INet_OutgoingMail *result = NULL;
	BPTR fh;
	if ((result = AllocVec(sizeof(INet_OutgoingMail), MEMF_ANY | MEMF_CLEAR)))
	{
		NewList((List *)&result->to);
		if ((fh = Open(filename, MODE_OLDFILE)))
		{
			ULONG filelength;
			if (Seek(fh, 0, OFFSET_END) != -1)
			{
				if ((filelength = Seek(fh, 0, OFFSET_BEGINNING)) != -1)
				{
					if (Read(fh, result, 28) != -1)
					{
						if ((result->header = AllocVec(result->HeaderSize, MEMF_ANY | MEMF_CLEAR)))
						{
							if ((result->text = AllocVec(result->TextSize, MEMF_ANY | MEMF_CLEAR)))
							{
								if ((result->totable = AllocVec(result->TableEntrys + result->TableSize, MEMF_ANY | MEMF_CLEAR)))
								{
									if (Read(fh, result->totable, result->TableEntrys + result->TableSize) != -1)
									{
										if (Read(fh, result->header, result->HeaderSize) != -1)
										{
											if (Read(fh, result->text, result->TextSize) != -1)
											{
												struct MainPortConfig *mainportconfig;
												if ((mainportconfig = getmainportconfig()))
												{
													struct INet_POP3Server *server = (struct INet_POP3Server *)mainportconfig->POP3ServerList.mlh_Head;
													ULONG counter;
													struct INet_SendMailHeaderEntry *entry;
													char *length	= result->totable;
													char *ptr		= result->totable + result->TableEntrys;
													char *stringptr;
													for (counter = 0; counter < result->TableEntrys; counter++)
													{
														if (*length)
														{
															if ((entry = AllocVec(sizeof(INet_SendMailHeaderEntry), MEMF_ANY | MEMF_CLEAR)))
															{
																if ((entry->string = AllocVec((ULONG)*length, MEMF_ANY | MEMF_CLEAR)))
																{
																	stringptr = entry->string;
																	while (*ptr)
																		*stringptr++ = *ptr++;
																	ptr++;
																}
																AddTail((List *)&result->to, (Node *)entry);
															}
														}
														length++;
													}
													uucp_sendmail(server, result);
												}
											}
										}
									}
								}
							}
						}
					}
					else
					{
						FreeVec(result);
						result = NULL;
					}
				}
			}
			Close(fh);
		}
	}
	return(result);
}
